@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 import qondor
-import logging, os, os.path as osp, pprint, shutil, uuid, re, json
+import logging, os, os.path as osp, pprint, shutil, uuid, re, json, sys
 from datetime import datetime
 from time import strftime
 from contextlib import contextmanager
@@ -52,30 +52,13 @@ def split_runcode_submitcode_file(filename):
     with open(filename, 'r') as f:
         return split_runcode_submitcode(f.readlines())
 
-def submit_python_job_file(filename, cli=False, njobsmax=None, run_args=None):
-    """
-    Builds submission clusters from a python job file
-    """
-    runcode, submitcode = split_runcode_submitcode_file(filename)
-    # Run the submitcode
-    # First create exec scope dict, with a few handy functions
-    session = Session(name=osp.basename(filename).replace('.py',''))
-    def submit_fn(*args, **kwargs):
-        cluster = Cluster(runcode, *args, session=session, run_args=run_args, **kwargs)
-        session.add_submission(cluster)
-    def submit_now_fn():
-        session.submit(cli)
-    exec_scope = {
-        'qondor' : qondor,
-        'session' : session,
-        'submit' : submit_fn,
-        'submit_now' : submit_now_fn,
-        'runcode' : runcode
-        }
-    logger.warning('Running submission code now')
-    exec(submitcode, exec_scope)
-    # Do a submit_now call in case it wasn't done yet (if submit_now was called in the submit code this is a no-op)
-    session.submit(cli)
+def _exec_in_scope(code, scope):
+    # if sys.version_info.major < 3:
+    #     if not code.endswith('\n'): code += '\n'
+    #     logger.warning('Python 2 style')
+    #     exec code in scope
+    # else:
+    exec(code, scope)
 
 class StopProcessing(Exception):
     """
@@ -83,34 +66,61 @@ class StopProcessing(Exception):
     """
     pass
 
+def submit_python_job_file(filename, cli=False, njobsmax=None, run_args=None, return_first_cluster=False):
+    """
+    Builds submission clusters from a python job file
+    """
+    runcode, submitcode = split_runcode_submitcode_file(filename)
+    # Run the submitcode
+    # First create exec scope dict, with a few handy functions
+    _first_cluster_ptr = [None]
+    session = Session(name=osp.basename(filename).replace('.py',''))
+    def htcondor_setting(key, value):
+        session.htcondor(key, value)
+    def submit_fn(*args, **kwargs):
+        kwargs.setdefault('session', session)
+        kwargs.setdefault('run_args', run_args)
+        cluster = Cluster(runcode, *args, **kwargs)
+        if return_first_cluster:
+            _first_cluster_ptr[0] = cluster
+            raise StopProcessing
+        session.add_submission(cluster, cli=cli, njobsmax=njobsmax)
+    def submit_now_fn():
+        session.submit(cli, njobsmax=njobsmax)
+    exec_scope = {
+        'qondor'     : qondor,
+        'session'    : session,
+        'htcondor'   : htcondor_setting,
+        'submit'     : submit_fn,
+        'submit_now' : submit_now_fn,
+        'runcode'    : runcode,
+        'run_args'   : run_args,
+        'cli'        : cli,
+        'njobsmax'   : njobsmax,
+        'return_first_cluster' : return_first_cluster
+        }
+    logger.warning('Running submission code now')
+    if return_first_cluster:
+        try:
+            _exec_in_scope(submitcode, exec_scope)
+        except StopProcessing:
+            pass
+        cluster = _first_cluster_ptr[0]
+        if cluster is None:
+            raise Exception('No cluster was submitted in the submit code')
+        return cluster
+    else:
+        _exec_in_scope(submitcode, exec_scope)
+        # Do a submit_now call in case it wasn't done yet (if submit_now was called in the submit code this is a no-op)
+        session.submit(cli, njobsmax=njobsmax)
+
+
 def get_first_cluster(filename):
     """
     Returns the first cluster that would be submitted if the python job file
     would be submitted
     """
-    runcode, submitcode = split_runcode_submitcode_file(filename)
-    cluster = [None]
-    def submit_fn(*args, **kwargs):
-        cluster[0] = Cluster(runcode, *args, **kwargs)
-        raise StopProcessing
-    def submit_now_fn():
-        session.submit(cli)
-    exec_scope = {
-        'cluster' : cluster,
-        'qondor' : qondor,
-        'submit' : submit_fn,
-        'submit_now' : submit_now_fn,
-        'runcode' : runcode
-        }
-    logger.warning('Running submission code now')
-    try:
-        exec(submitcode, exec_scope)
-    except StopProcessing:
-        pass
-    cluster = cluster[0]
-    if cluster is None:
-        raise Exception('No cluster was submitted in the submit code')
-    return cluster
+    return submit_python_job_file(filename, return_first_cluster=True)
 
 
 RUN_ENVS = {
@@ -132,9 +142,6 @@ RUN_ENVS = {
         ],
     }
 
-class SHFile():
-    pass
-
 
 def get_default_sub(submission_time=None):
     """
@@ -175,6 +182,21 @@ def get_default_sub(submission_time=None):
         pass
     return sub
 
+def update_sub(sub, other):
+    """
+    Merges a submission dict, attempting not to overwrite some keys but rather append them
+    """
+    # First copy all
+    r = dict(sub, **other)
+    # Merge some things more carefully:
+    # Environment
+    r['environment'] = dict(sub.get('environment', {}), **other.get('environment', {}))
+    # Input files
+    files = []
+    if 'transfer_input_files' in sub: files += sub['transfer_input_files'].split(',')
+    if 'transfer_input_files' in other: files += other['transfer_input_files'].split(',')
+    if files: r['transfer_input_files'] = ','.join(files)
+    return r
 
 class Session(object):
     """
@@ -188,6 +210,29 @@ class Session(object):
         self._created_python_module_tarballs = {}
         self._i_seutils_tarball = 0
         self.submittables = []
+        self._njobs_submitted = 0
+        self.htcondor_settings = get_default_sub(self.submission_time)
+        self.htcondor_settings['+QondorRundir']  =  '"' + self.rundir + '"'
+        self._fixed_cmsconnect_specific_settings = False
+
+    def htcondor(self, key, value):
+        """
+        Shortcut to add htcondor settings
+        """
+        self.htcondor_settings[key] = value
+
+    def fix_cmsconnect_specific_settings_once(self, cli):
+        """
+        Potentially process cmsconnect specific settings
+        Changes self.htcondor_settings once
+        """
+        if self._fixed_cmsconnect_specific_settings: return
+        self._fixed_cmsconnect_specific_settings = True
+        blacklist = self.htcondor_settings.pop('cmsconnect_blacklist', None)
+        whitelist = self.htcondor_settings.pop('cmsconnect_whitelist', None)
+        if os.uname()[1] == 'login.uscms.org' or os.uname()[1] == 'login-el7.uscms.org':
+            qondor.logger.warning('Detected CMS Connect; loading specific settings')
+            cmsconnect_settings(self.htcondor_settings, cli=cli, blacklist=blacklist, whitelist=whitelist)
 
     def dump_seutils_cache(self):
         """
@@ -214,24 +259,35 @@ class Session(object):
                 # Add the tarball as input file for this cluster
                 cluster.transfer_files['_packagetarball_{}'.format(package)] = self._created_python_module_tarballs[package]
 
-    def make_sub(self, cluster):
-        sub = get_default_sub(self.submission_time)
+    def make_sub(self, cluster, cli):
+        # Base off of global settings only for python-binding mode
+        # For the condor_submit cli method, it's better to just write the global keys once at the top of the file
+        # Little unsafe, if you modiy these 'global' settings in one job they propegate to all the next ones,
+        # but otherwise submission of many jobs becomes very slow
+        sub = {}
         sub['executable'] =  osp.basename(cluster.sh_entrypoint_filename)
-        sub['+QondorRundir']  =  '"' + self.rundir + '"'
+        sub['environment'] = {}
         sub['environment']['QONDORICLUSTER'] = str(cluster.i_cluster)
         sub['environment'].update(cluster.env)
         # Overwrite htcondor keys defined in the preprocessing
         sub.update(cluster.htcondor)
         # Flatten files into a string, excluding files on storage elements
-        print(cluster.transfer_files.values())
         transfer_files = self.transfer_files + [f for f in cluster.transfer_files.values() if not seutils.has_protocol(f)]
         if len(transfer_files) > 0:
             sub['transfer_input_files'] = ','.join(transfer_files)
+        sub = update_sub(sub, cluster.htcondor)
+        # Plugin the global and cmsconnect settings in now
+        self.fix_cmsconnect_specific_settings_once(cli)
+        sub = update_sub(self.htcondor_settings, sub)
         logger.info('Prepared submission dict for cluster %s:\n%s', cluster.i_cluster, pprint.pformat(sub))
         return sub
 
-    def add_submission(self, cluster, cli=True, njobs=1):
-        qondor.utils.check_proxy()
+    def add_submission(self, cluster, cli=True, njobs=1, njobsmax=None):
+        if njobsmax: njobs = min(njobs, njobsmax - self._njobs_submitted)
+        if njobsmax and njobs == 0:
+            logger.debug('Not adding submission for cluster %s - reached njobsmax %s', cluster.i_cluster, njobsmax)
+            return
+        self._njobs_submitted += njobs
         qondor.utils.create_directory(self.rundir)
         cluster.rundir = self.rundir
         # Possibly create tarballs out of required python packages
@@ -243,11 +299,12 @@ class Session(object):
         cluster.sh_entrypoint_to_file()
         cluster.scope_to_file()
         # Compile the submission dict
-        sub = self.make_sub(cluster)
+        sub = self.make_sub(cluster, cli)
         # Add it to the submittables
         self.submittables.append((sub, njobs))
 
     def submit_pythonbindings(self, njobsmax=None):
+        qondor.utils.check_proxy()
         if not self.submittables: return
         import htcondor
         if njobsmax is None: njobsmax = 1e7
@@ -260,8 +317,8 @@ class Session(object):
         with qondor.utils.switchdir(self.rundir):
             with qondor.schedd._transaction(schedd) as transaction:
                 submit_object = htcondor.Submit()
-                for sub, njobs in self.submittables:
-                    sub = sub.copy() # Keep original dict intact
+                for sub_orig, njobs in self.submittables:
+                    sub = sub_orig.copy() # Keep original dict intact? Global settings already contained
                     sub['environment'] = qondor.schedd.format_env_htcondor(sub['environment'])
                     njobs = min(njobs, n_jobs_todo)
                     n_jobs_todo -= njobs
@@ -270,12 +327,19 @@ class Session(object):
                         submit_object[key] = sub[key]
                     new_ads = []
                     cluster_id = int(submit_object.queue(transaction, njobs, new_ads)) if not qondor.DRYMODE else 0
-                    logger.info('Submitted %s jobs to cluster %s', len(new_ads) if not qondor.DRYMODE else njobs, cluster_id)
+                    logger.info(
+                        'Submitted %s jobs for i_cluster %s to htcondor cluster %s',
+                        len(new_ads) if not qondor.DRYMODE else njobs,
+                        sub_orig['environment']['QONDORICLUSTER'], cluster_id
+                        )
                     ads.extend(new_ads)
+                    if n_jobs_todo == 0: break
+        logger.info('Summary: Submitted %s jobs to cluster %s', n_jobs_total, cluster_id)
         # Clear submittables
         self.submittables = []
 
     def submit_cli(self, njobsmax=None):
+        qondor.utils.check_proxy()
         if not self.submittables: return
         if njobsmax is None: njobsmax = 1e7
         n_jobs_summed = sum([ njobs for _, njobs in self.submittables ])
@@ -289,23 +353,39 @@ class Session(object):
             self.rundir,
             'qondor_{}-{}.jdl'.format(get_cluster_nr(self.submittables[0][0]), get_cluster_nr(self.submittables[-1][0]))
             )
+        jdl_contents = []
+        # Write the settings per job
+        for sub, njobs in self.submittables:
+            njobs = min(njobs, n_jobs_todo)
+            n_jobs_todo -= njobs
+            jdl_contents.append('# Cluster {}'.format(sub['environment']['QONDORICLUSTER']))
+            # Dump the submission to a jdl file
+            for key in sub.keys():
+                val = sub[key]
+                if key.lower() == 'environment': val = qondor.schedd.format_env_htcondor(val)
+                jdl_contents.append('{} = {}'.format(key, val))
+            jdl_contents.append('queue {}\n'.format(njobs))
+        # Dump to file
+        jdl_contents = '\n'.join(jdl_contents)
         with qondor.utils.openfile(jdl_file, 'w') as jdl:
-            for sub, njobs in self.submittables:
-                njobs = min(njobs, n_jobs_todo)
-                n_jobs_todo -= njobs
-                jdl.write('# Cluster {}\n'.format(sub['environment']['QONDORICLUSTER']))
-                # Dump the submission to a jdl file
-                for key in sub.keys():
-                    val = sub[key]
-                    if key.lower() == 'environment': val = qondor.schedd.format_env_htcondor(val)
-                    jdl.write('{} = {}\n'.format(key, val))
-                jdl.write('queue {}\n\n'.format(njobs))
-
-            if qondor.DRYMODE: logger.info('Compiled %s:\n%s', jdl_file, jdl.text)
+            jdl.write(jdl_contents)
+        logger.info('Compiled %s:\n%s', jdl_file, jdl_contents)
         # Run the actual submit command
         with qondor.utils.switchdir(self.rundir):
-            qondor.utils.run_command(['condor_submit', osp.basename(jdl_file)])
+            output = qondor.utils.run_command(['condor_submit', osp.basename(jdl_file)])
+        # Clear the submittables
         self.submittables = []
+        # Get some info from the condor_submit command, if not dry mode
+        if qondor.DRYMODE:
+            logger.info('Summary: Submitted %s jobs to cluster 0', n_jobs_total)
+        else:
+            match = re.search(r'(\d+) job\(s\) submitted to cluster (\d+)', '\n'.join(output))
+            if match:
+                logger.info('Submitted %s jobs to cluster_id %s', match.group(1), match.group(2))
+            else:
+                logger.error(
+                    'condor_submit exited ok but could not determine where and how many jobs were submitted'
+                    )
 
     def submit(self, cli, *args, **kwargs):
         """
@@ -336,7 +416,7 @@ class Cluster(object):
         self.runcode = runcode
         self.env = {} if env is None else env
         self.scope = {} if scope is None else scope
-        self.htcondor = [] if htcondor is None else htcondor
+        self.htcondor = {} if htcondor is None else htcondor
         self.run_args = run_args
         if qondor.utils.is_string(run_env): run_env = RUN_ENVS[run_env]
         self.run_env = run_env
@@ -468,245 +548,19 @@ class Cluster(object):
         logger.info('Parsed the following .sh entrypoint for cluster %s:\n%s', self.i_cluster, sh)
         return sh
 
-    def submit(self, *args, session=None, **kwargs):
+    def submit(self, *args, **kwargs):
         """
         Like Session.submit(cluster, *args, **kwargs), but then initiated from the cluster object.
-        If session is None, a clean session is started.
+        If session is not provided as a keyword argument, a clean session is started.
         """
-        if session is None: session = Session()
+        session = kwargs.pop('session', Session()) # Make new session if not set by keyword
         session.submit(self, *args, **kwargs)
 
 
-class BaseSubmitter(object):
+def cmsconnect_get_all_sites():
     """
-    Base class for Submitter: Translates user input to one of the following:
-    - a submission dict + the number of jobs to submit
-    - a submission dict + a list of items
-    - a submission dict + a list of chunks
-    , and creates a directory with the needed input files for the job
+    Reads the central config for cmsconnect to determine the list of all available sites
     """
-
-    def __init__(self, python_script_args=None):
-        super(BaseSubmitter, self).__init__()
-        self.transfer_files = []
-        self.python_script_args = python_script_args
-        self._created_python_module_tarballs = []
-
-    # _________________________________________________________
-    # Methods to prep submission dict
-
-    def format_for_htcondor_interface(self, shfile, preprocessor):
-        """
-        Formats a submission dict and possibly an extra argument (njobs/items/chunks).
-        Returns a tuple (submissiondict, extra_settings), where the extra_settings is
-        a dictionary with the following possible keys:
-        - 'items' in case there were items found in the preprocessing
-        - 'rootfile_chunks' in case there were rootfile_chunks found in the preprocessing
-        - 'njobs' in case the njobs variable was set in the preprocessing
-        - 'njobsmax'?
-        """
-        extra_submission_settings = {}
-        sub = qondor.schedd.get_default_sub()
-        sub['executable'] =  osp.basename(shfile)
-        sub['+QondorRundir']  =  '"' + self.rundir + '"'
-        sub['environment']['QONDORISCOPE'] = str(preprocessor.subscope_index)
-        # Overwrite htcondor keys defined in the preprocessing
-        sub.update(preprocessor.htcondor)
-        # Flatten files into a string, excluding files on storage elements
-        transfer_files = self.transfer_files + [f for f in preprocessor.files.values() if not seutils.has_protocol(f)]
-        if len(transfer_files) > 0:
-            sub['transfer_input_files'] = ','.join(transfer_files)
-        # Otherwise continue processing items
-        if preprocessor.items and preprocessor.rootfile_chunks:
-            raise ValueError(
-                'Both regular items and rootfiles-split-by-entries are specified; '
-                'This is not supported.'
-                )
-        elif preprocessor.items:
-            logger.info('Items:\n%s', pprint.pformat(preprocessor.items))
-            extra_submission_settings['items'] = preprocessor.items
-        elif preprocessor.rootfile_chunks:
-            logger.info('Items as rootfile chunks:\n%s', pprint.pformat(preprocessor.rootfile_chunks))
-            extra_submission_settings['rootfile_chunks'] = preprocessor.rootfile_chunks
-        else:
-            # Simple case with no items; just use the njobs variable with a default of 1
-            if 'njobs' in preprocessor.variables:
-                extra_submission_settings['njobs'] = int(preprocessor.variables['njobs'])
-        return sub, extra_submission_settings
-
-    # _________________________________________________________
-    # Prep functions for files needed for the job
-
-    def tar_python_module(self, module_name):
-        """
-        Creates a tarball of a python module that is needed inside the job
-        """
-        if module_name in self._created_python_module_tarballs:
-            logger.debug('Not tarring up %s again, tarball already created', module_name)
-            return
-        logger.info('Creating tarball for python module %s', module_name)
-        import importlib
-        module = importlib.import_module(module_name)
-        tarball = qondor.utils.tarball_python_module(
-            module,
-            outdir = self.rundir,
-            )
-        self.transfer_files.append(tarball)
-        self._created_python_module_tarballs.append(module_name)
-
-    def make_rundir(self):
-        """
-        Creates a directory in from which to submit the job
-        """
-        self.rundir = osp.abspath('qondor_{0}_{1}'.format(
-            self.python_name,
-            strftime(qondor.TIMESTAMP_FMT)
-            ))
-        qondor.utils.create_directory(
-            self.rundir,
-            must_not_exist=True,
-            )
-
-    def dump_ls_cache_file(self):
-        """
-        Creates the ls cache file for the job
-        """
-        cache_file = osp.join(self.rundir, qondor.Preprocessor.LS_CACHE_FILE)
-        qondor.Preprocessor.dump_ls_cache(cache_file)
-        self.transfer_files.append(cache_file)
-
-    def dump_rootcache(self):
-        """
-        Dumps the current state of the seutils root cache to a file to be used in the job
-        """
-        if seutils.root.USE_CACHE:
-            rootcache_file = osp.join(self.rundir, 'rootcache.tar.gz')
-            if not qondor.DRYMODE: seutils.root.cache_to_file(rootcache_file)
-            self.transfer_files.append(rootcache_file)
-
-    def iter_submissions(self):
-        """
-        Does all the prep work for the submission directory, and then
-        loops over all scopes and yields (submission_dict, extra_settings) as
-        returned by self.format_for_htcondor_interface
-        """
-        # First do all prep
-        at_least_one_job_submitted = False
-        try:
-            self.make_rundir()
-            self.create_python_file()
-            self.dump_ls_cache_file()
-            self.dump_rootcache()
-            # Loop over all 'scopes'
-            # If there are no subscopes, this is just a len(1) iterator of the preprocessing
-            for i_scope, preprocessor in enumerate(self.preprocessing.scopes()):
-                # Create tarballs for local python modules
-                for package, install_instruction in preprocessor.pip:
-                    if install_instruction == 'editable-install':
-                        self.tar_python_module(package)
-                # Create the bash script entry point for this job
-                shfile = osp.join(self.rundir, '{}_{}.sh'.format(self.python_name, i_scope))
-                SHFile(preprocessor, self.python_script_args).to_file(shfile)
-                # Submit jobs to htcondor
-                sub, extra_settings = self.format_for_htcondor_interface(shfile, preprocessor)
-                yield sub, extra_settings, preprocessor
-                at_least_one_job_submitted = True
-        except StopIteration:
-            pass
-        except Exception as e:
-            logger.error('Exception %s was raised', e)
-            if not at_least_one_job_submitted:
-                logger.error('Error during submission; cleaning up %s', self.rundir)
-                if osp.isdir(self.rundir) and not qondor.DRYMODE: shutil.rmtree(self.rundir)
-            raise
-
-    def submissions(self):
-        return list(self.iter_submissions())
-
-
-class Submitter(BaseSubmitter):
-    """
-    Standard Submitter based on a python file.
-    Upon running `.submit()`, will create a new directory,
-    transfer all relevant files from the job, and start
-    running.
-    """
-    def __init__(self, python_file, python_script_args=None):
-        super(Submitter, self).__init__(python_script_args=python_script_args)
-        self.original_python_file = osp.abspath(python_file)
-        self.python_base = osp.basename(self.original_python_file)
-        self.python_name = self.python_base.replace('.py','')
-        self.preprocessing = qondor.Preprocessor(self.original_python_file)
-
-    def create_python_file(self):
-        self.python_file = osp.join(self.rundir, self.python_base)
-        qondor.utils.copy_file(self.original_python_file, self.python_file)
-        self.transfer_files.append(self.python_file)
-
-
-class SubmitterPreproc(BaseSubmitter):
-    """
-    Submits from a preprocessor object directly.
-    """
-    def __init__(self, preprocessing):
-        pass
-
-
-class CodeSubmitter(BaseSubmitter):
-    """
-    Like submitter, but rather than being instantiated from a python file
-    it is instantiated from python code in a string
-    """
-    def __init__(self, python_code, preprocessing_code=None, name='', dry=False):
-        super(CodeSubmitter, self).__init__(dry)
-        self.python_code = python_code.split('\n') if qondor.utils.is_string(python_code) else python_code
-        if preprocessing_code is None: preprocessing_code = []
-        self.preprocessing_code = preprocessing_code.split('\n') if qondor.utils.is_string(preprocessing_code) else preprocessing_code
-        self.python_name = name if name else 'fromcode'
-        self.preprocessing = qondor.Preprocessor.from_lines(self.preprocessing_code)
-
-    def create_python_file(self):
-        self.python_file = osp.join(self.rundir, 'pythoncode.py')
-        logger.info('Writing %s lines of code to %s', len(self.python_code), self.python_file)
-        with open(self.python_file, 'w') as f:
-            f.write('\n'.join(['#$ ' + l for l in self.preprocessing_code]))
-            f.write('\n')
-            f.write('\n'.join(self.python_code))
-        self.transfer_files.append(self.python_file)
-        self.preprocessing.filename = self.python_file
-
-    def run_local_exec(self):
-        logger.warning('Doing exec() on the following code:\n%s', '\n'.join(self.python_code))
-        if '__file__' in self.python_code:
-            logger.error(
-                'Note that "__file__" will crash, as it is not defined in exec(). '
-                'Use .run_local() instead.'
-                )
-        logger.warning('Output:')
-        exec('\n'.join(self.python_code))
-
-    def run_local(self):
-        python_file = 'temp-{0}.py'.format(uuid.uuid4())
-        logger.info('Put python code in {0} and running:'.format(python_file))
-        try:
-            with open(python_file, 'w') as f:
-                f.write('\n'.join(['#$ ' + l for l in self.preprocessing_code]))
-                f.write('\n')
-                f.write('\n'.join(self.python_code))
-            return qondor.utils.run_command(['python', python_file])
-        finally:
-            logger.info('Removing %s', python_file)
-            os.remove(python_file)
-
-
-def cmsconnect_settings(sub, preprocessor=None, cli=False):
-    """
-    Adds special cmsconnect settings to submission dict in order to submit
-    on cmsconnect via the python bindings, or in case of submitting by the
-    command line, to set the DESIRED_Sites key.
-    Modifies the dict in place.
-    """
-    # Read the central config for cmsconnect
     try:
         from configparser import RawConfigParser # python 3
     except ImportError:
@@ -716,36 +570,52 @@ def cmsconnect_settings(sub, preprocessor=None, cli=False):
     cfg.read('/etc/ciconnect/config.ini')
     sites = cfg.get('submit', 'DefaultSites').split(',')
     all_sites = set(sites)
+    return all_sites
+
+def cmsconnect_settings(sub, blacklist=None, whitelist=None, cli=False):
+    """
+    Adds special cmsconnect settings to submission dict in order to submit
+    on cmsconnect via the python bindings, or in case of submitting by the
+    command line, to set the DESIRED_Sites key.
+    Modifies the dict in place.
+    """
+    all_sites = cmsconnect_get_all_sites()
+
     # Check whether the user whitelisted or blacklisted some sites
-    if preprocessor:
+    desired_sites = None
+    if blacklist or whitelist:
         import fnmatch
         blacklisted = []
         whitelisted = []
         # Build the blacklist
-        if 'sites_blacklist' in preprocessor.variables:
-            for blacksite in preprocessor.variables['sites_blacklist'].split():
-                for site in sites:
-                    if fnmatch.fnmatch(site, blacksite):
+        if blacklist:
+            for blacksite_pattern in blacklist:
+                for site in all_sites:
+                    if fnmatch.fnmatch(site, blacksite_pattern):
                         blacklisted.append(site)
         # Build the whitelist
-        if 'sites_whitelist' in preprocessor.variables:
-            for whitesite in preprocessor.variables['sites_whitelist'].split():
-                for site in sites:
-                    if fnmatch.fnmatch(site, whitesite):
+        if whitelist:
+            for whitesite_pattern in whitelist:
+                for site in all_sites:
+                    if fnmatch.fnmatch(site, whitesite_pattern):
                         whitelisted.append(site)
+        # Convert to list and sort
         blacklisted = list(set(blacklisted))
         blacklisted.sort()
         whitelisted = list(set(whitelisted))
         whitelisted.sort()
         logger.info('Blacklisting: %s', ','.join(blacklisted))
         logger.info('Whitelisting: %s', ','.join(whitelisted))
-        sites = list( (set(sites) - set(blacklisted)).union(set(whitelisted)) )
-        sites.sort()
-    logger.info('Submitting to sites: %s', ','.join(sites))
+        desired_sites = list( (set(all_sites) - set(blacklisted)).union(set(whitelisted)) )
+        desired_sites.sort()
+
     # Add a plus only if submitting via .jdl file
     addplus = lambda key: '+' + key if cli else key
-    if all_sites != set(sites):
-        sub[addplus('DESIRED_Sites')] = '"' + ','.join(sites) + '"'
+    if desired_sites:
+        logger.info('Submitting to desired sites: %s', ','.join(desired_sites))
+        sub[addplus('DESIRED_Sites')] = '"' + ','.join(desired_sites) + '"'
+    else:
+        logger.info('Submitting to all sites: %s', ','.join(all_sites))
     if not cli:
         sub[addplus('ConnectWrapper')] = '"2.0"'
         sub[addplus('CMSGroups')] = '"/cms,T3_US_FNALLPC"'
